import React, { memo, createContext, useContext } from 'react';
import { NavigationContext } from 'react-navigation';
import { NativeNavigatorTransitions, NativeNavigatorHeaderModes, NativeNavigatorModes } from './types';
import NativeSceneView from './NativeSceneView';
import NativeStackScene from './NativeStackScene';
import NativeHeader from './NativeHeader';
import NativeStackSceneContainer from './NativeStackSceneContainer';
// 'willBlur' event won't emit during the closing transition.
// NativeNavigationClosingStateContext is convenient for some cases when you want to get the closing state for a scene.
export const NativeNavigationClosingStateContext = createContext(false);
export default memo(function NativeScene(props) {
    var _a, _b, _c;
    const { route, descriptor, closing, isSplitPrimary, mode, screenProps, onDidFocus, onDidBlur } = props;
    const { key } = route;
    const closingContextState = useContext(NativeNavigationClosingStateContext);
    if (!descriptor) {
        return null;
    }
    const { options, navigation } = descriptor;
    let headerMode = options.headerHidden
        ? NativeNavigatorHeaderModes.None
        : props.headerMode;
    if (mode === NativeNavigatorModes.Stack) {
        headerMode = (headerMode !== null && headerMode !== void 0 ? headerMode : NativeNavigatorHeaderModes.Auto);
    }
    else if (mode === NativeNavigatorModes.Split) {
        headerMode = NativeNavigatorHeaderModes.None; // split navigator do not support native header
    }
    else {
        headerMode = (headerMode !== null && headerMode !== void 0 ? headerMode : NativeNavigatorHeaderModes.None);
    }
    return (<NativeStackScene key={key} transition={((_b = (_a = navigation.dangerouslyGetParent()) === null || _a === void 0 ? void 0 : _a.state) === null || _b === void 0 ? void 0 : _b.isTransitioning) ? (_c = options.transition, (_c !== null && _c !== void 0 ? _c : NativeNavigatorTransitions.Default)) : NativeNavigatorTransitions.None} isSplitPrimary={isSplitPrimary} gestureEnabled={options.gestureEnabled !== false} translucent={options.translucent === true} transparent={options.transparent === true} style={options.cardStyle} statusBarStyle={options.statusBarStyle} statusBarHidden={options.statusBarHidden} closing={closing} onDidFocus={onDidFocus} onDidBlur={onDidBlur} route={route}>
      <NativeStackSceneContainer>
        <NativeNavigationClosingStateContext.Provider value={closing || closingContextState}>
          <NativeSceneView screenProps={screenProps} navigation={navigation} component={descriptor.getComponent()}/>
        </NativeNavigationClosingStateContext.Provider>
        {headerMode === NativeNavigatorHeaderModes.Auto ? (<NavigationContext.Provider value={navigation}>
            <NativeHeader options={options} route={route}/>
          </NavigationContext.Provider>) : null}
      </NativeStackSceneContainer>
    </NativeStackScene>);
});
//# sourceMappingURL=NativeScene.js.map