import { StackRouter, NavigationActions } from 'react-navigation';
export default function SplitRouter(routeConfigs, config) {
    const stackRouter = StackRouter(routeConfigs, config);
    const { getStateForAction } = stackRouter;
    stackRouter.getStateForAction = (action, state) => {
        if (state) {
            if (action.type === NavigationActions.NAVIGATE) {
                const newState = getStateForAction(action, state);
                // StackRouter 对非 active 的子路由进行 NAVIGATE 操作时会主动将该子路由切换成 active
                // 如果在 SplitRouter 中对 primary route 的子路由做 NAVIGATE 导航会导致右侧路由退出，交互不符合预期
                if (newState && newState.index > -1 && newState.index < state.index) {
                    const newRoutes = state.routes.slice();
                    newRoutes.splice(newState.index, 1, newState.routes[newState.index]);
                    return Object.assign(Object.assign({}, newState), { index: state.index, routes: newRoutes });
                }
                else {
                    return newState;
                }
            }
            else if (action.type === NavigationActions.BACK && action.key) {
                const newState = getStateForAction(action, state);
                // StackRouter 通过指定 key 的 action 对非 active 的子路由进行 BACK 导航时会将 active index 切换到该子路由
                // 如果 SplitRouter 中对 primary route 的子路由做 BACK 导航会导致右侧路由退出，交互不符合预期
                if (newState && newState.index < newState.routes.length - 1) {
                    return Object.assign(Object.assign({}, newState), { index: newState.routes.length - 1 });
                }
                else {
                    return newState;
                }
            }
        }
        return getStateForAction(action, state);
    };
    return stackRouter;
}
//# sourceMappingURL=SplitRouter.js.map